##
## Replication Rapach & Strauss (2008) - JAE
##
## Standard GARCH(1,1) expanding window model assuming normal
##
model = c("sGARCH") ## Model choices: c("sGARCH", "eGARCH", "gjrGARCH", "apARCH", "csGARCH")
archm = c(FALSE)
distrib = c("norm") ## Distributional assumptions: c("norm", "std", "ged", "sged", "snorm", "sstd", "jsu", "ghyp")
##
model.name = if(archm==TRUE){paste("exp", model,"M",distrib,R,"full", sep="-")} else{paste("exp", model, distrib,R,"full", sep="-")}
##
############### Make the Loop ##############
##
options(max.print = 100000)
garch.fit <- list()
forc <- list()
out.forc <- list()
vol.forc <- list()
spec <- list()
arima.fit <- list()
##
for (i in seq(step, R, step)) {
##  
  arima.fit[[i]] <- auto.arima(rets[(1):(fixed.smpl+i-step)], stepwise = FALSE, allowmean = TRUE)
##  
  spec[[i]] <- ugarchspec(variance.model = list(model = paste(model), garchOrder = c(1,1)),
                     mean.model = list(armaOrder = c(arimaorder(arima.fit[[i]])[1], 
                                                     arimaorder(arima.fit[[i]])[3]), 
                                       arfima = FALSE, include.mean = TRUE, 
                                       archm = archm, archpow = 2),
                     distribution.model = paste(distrib))
## 
  garch.fit[[i]] <- ugarchfit(rets[(1):(fixed.smpl+i-step)], spec = spec[[i]], 
                              solver = "hybrid", out.sample=0) 
  forc[[i]] = ugarchforecast(garch.fit[[i]], n.ahead=step)
  # print(garch.fit[[i]])
  print(forc[[i]])
  #
  for (k in seq(1, step, 1)) {
    vol.forc[[k]] <- forc[[i]]@forecast$sigmaFor[[k]]
    cat(vol.forc[[k]],file=paste(model.name, step, paste(commodity, ".txt", sep=""), sep="-"), sep="\n", append=TRUE)
    print(vol.forc[[k]])
    }
  print(i)
  print(model.name)
}
#
######################### Prepare the data for LossVol ###########3
options(max.print = 100000)
forecasts = read.table(paste(model.name, step, paste(commodity, ".txt", sep=""), sep="-"), header=F) # Load the data
######################### Prepare the data for LossVol ###########3
##
Loss.Vol_AE1 <- LossVol(which = "AE1", realized=actual1, evaluated = forecasts^2)
##
Loss.Vol_AE2 <- LossVol(which = "AE2", realized=actual1, evaluated = forecasts^2)
##
Loss.Vol_QLIKE <- LossVol(which = "QLIKE", realized=sqrt(actual1), evaluated = abs(forecasts) )
##
Loss.Vol_SE1 <- LossVol(which = "SE1", realized=actual1, evaluated = forecasts^2 )
##
Loss.Vol_SE2 <- LossVol(which = "SE2", realized=actual1, evaluated = forecasts^2)
##
Loss.Vol_R2LOG <- LossVol(which = "R2LOG", realized=sqrt(actual1), evaluated = abs(forecasts) )
##
Loss.Vol_HMSE <- (LossVol(which = "SE1", realized=actual1, evaluated = forecasts^2)) / (forecasts^2)^2
##
Loss.Vol_HMAE <- (LossVol(which = "AE1", realized=actual1, evaluated = forecasts^2)) / forecasts^2
##
#### Saving the results in the working dyrectory ######
##
options(max.print = 100000)
##
colnames(Loss.Vol_AE1) <- model.name
colnames(Loss.Vol_AE2) <- model.name
colnames(Loss.Vol_SE1) <- model.name
colnames(Loss.Vol_SE2) <- model.name
colnames(Loss.Vol_QLIKE) <- model.name
colnames(Loss.Vol_R2LOG) <- model.name
colnames(Loss.Vol_HMSE) <- model.name
colnames(Loss.Vol_HMAE) <- model.name
##
out.lossvol_AE1 <- capture.output(Loss.Vol_AE1)
cat(out.lossvol_AE1, file=paste(model.name,"Loss.Vol_AE1", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
out.lossvol_AE2 <- capture.output(Loss.Vol_AE2)
cat(out.lossvol_AE2, file=paste(model.name,"Loss.Vol_AE2", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
out.lossvol_SE1 <- capture.output(Loss.Vol_SE1)
cat(out.lossvol_SE1, file=paste(model.name,"Loss.Vol_SE1", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
out.lossvol_SE2 <- capture.output(Loss.Vol_SE2)
cat(out.lossvol_SE2, file=paste(model.name,"Loss.Vol_SE2", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
out.lossvol_QLIKE <- capture.output(Loss.Vol_QLIKE)
cat(out.lossvol_QLIKE, file=paste(model.name,"Loss.Vol_QLIKE", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
out.lossvol_R2LOG <- capture.output(Loss.Vol_R2LOG)
cat(out.lossvol_R2LOG, file=paste(model.name,"Loss.Vol_R2LOG", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
out.lossvol_HMSE <- capture.output(Loss.Vol_HMSE)
cat(out.lossvol_HMSE, file=paste(model.name,"Loss.Vol_HMSE", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
out.lossvol_HMAE <- capture.output(Loss.Vol_HMAE)
cat(out.lossvol_HMAE, file=paste(model.name,"Loss.Vol_HMAE", step, "retsq", paste(commodity, ".txt", sep=""), sep = "-"),sep="\n",append=TRUE)
##
#### END #####